#!/bin/bash

BLINDS_NUMBER=$1
FORMATTED_BLINDS_NUMBER=$(printf "%02d" $BLINDS_NUMBER)
let FIFO_NUMBER=BLINDS_NUMBER+10

if [ $# -lt 1 ];then
  echo "Add the number of the blind (1-9) as parameter"
  exit 1
fi

bluetooth_address="BLUETOOTH_ADDRESS_BLIND_$FORMATTED_BLINDS_NUMBER"

if [ -z "${!bluetooth_address}" ];then
  echo "Set bluetooth address of blind in environment variable BLUETOOTH_ADDRESS_BLIND_$FORMATTED_BLINDS_NUMBER"
  exit 1
fi

BLUETOOTH_ADDRESS_BLIND=${!bluetooth_address}

if [ ! -e /tmp/blinds_$FIFO_NUMBER.fifo ];then
  mkfifo /tmp/blinds_$FIFO_NUMBER.fifo
  chmod 666 /tmp/blinds_$FIFO_NUMBER.fifo
fi

# connect the fifo with file descriptor
eval "exec $FIFO_NUMBER<>/tmp/blinds_${FIFO_NUMBER}.fifo"

final_position="FINAL_LOW_POSITION_$FORMATTED_BLINDS_NUMBER"

FINAL_LOW_POSITION=${!final_position}
DIFFERENCE=16
SMALL_DIFFERENCE=5

function get_state_of_blinds() {
  ./synchronize.sh ./blindsControl.exp ${BLUETOOTH_ADDRESS_BLIND} getheight > /dev/shm/getheight_${BLUETOOTH_ADDRESS_BLIND}.txt 2>&1
  cat /dev/shm/position_${BLUETOOTH_ADDRESS_BLIND}.txt
}

function set_height_of_blinds() {
  new_position=$1

  if [ $new_position -lt 0 ];then
    new_position=0
  fi

  if [ $new_position -gt $FINAL_LOW_POSITION ];then
    new_position=$FINAL_LOW_POSITION
  fi

  echo "new_position: $new_position"

  ./synchronize.sh ./blindsControl.exp ${BLUETOOTH_ADDRESS_BLIND} setheight $new_position > /dev/shm/setheight_${BLUETOOTH_ADDRESS_BLIND}.txt 2>&1
}

last_position=$(get_state_of_blinds)
moving_up=0
moving_down=0

echo "last_position: $last_position"

# read data from the fifo
while true; do

  read -t 2.5 -u $FIFO_NUMBER command

  if [ $? -eq 0 ];then

    position=$(get_state_of_blinds)
    echo "Command: $command - position: $position"

    if [ "$command" == "up" ];then
      if [ $position -ne 0 ];then
        if [ $moving_up -eq 0 ] && [ $moving_down -eq 0 ];then
          moving_up=1
          echo "Start moving blinds up"
          last_position=$position
          let new_position=position-SMALL_DIFFERENCE
          set_height_of_blinds $new_position

        else
          echo "stopping blinds"
          moving_up=0
        fi
      fi
      moving_down=0
    elif [ "$command" == "down" ];then
      if [ $position -lt $FINAL_LOW_POSITION ];then
        if [ $moving_up -eq 0 ] && [ $moving_down -eq 0 ];then
          moving_down=1
          echo "Start moving blinds down"
          last_position=$position
          let new_position=position+SMALL_DIFFERENCE
          set_height_of_blinds $new_position

        else
          echo "stopping blinds"
          moving_down=0
        fi
      fi
      moving_up=0
    fi
  else
    # nothing read from remote control

    if [ $moving_up -eq 1 ];then
      position=$(get_state_of_blinds)
      echo "Checking position of blinds: $position"

      if [ $last_position != $position ]; then
        last_position=$position

        if [ $position -eq 0 ];then
          echo "moving blinds up finished"
          moving_up=0
          stop_blinds=0
        else
          echo "moving blinds up again"
          let new_position=position-DIFFERENCE
          set_height_of_blinds $new_position
        fi
      fi
    elif [ $moving_down -eq 1 ];then
      position=$(get_state_of_blinds)
      echo "Checking position of blinds: $position"

      if [ $last_position != $position ]; then
        last_position=$position

        if [ $position -ge $FINAL_LOW_POSITION ];then
          echo "moving blinds down finished"
          moving_down=0
          stop_blinds=0
        else
          echo "moving blinds down again"
          let new_position=position+DIFFERENCE
          set_height_of_blinds $new_position
        fi
      fi
    fi

  fi
done
